package net.pricefx.common.apibuilder.calculationflowaction;

import net.pricefx.common.apibuilder.Invocations;

/**
 * This action starts calculation of selected rebate record set records. It's a
 * programmatic version of the same action available in UI. As such it allows
 * user to schedule the action in calculation flow formula.
 */
public class RebateRecordAction<ATTACHEDTO extends Help> extends Action {

	private String rrscLabel;
	private Boolean calculate;
	private ATTACHEDTO backLink;
	
	public RebateRecordAction(ATTACHEDTO backLink, Invocations<MethodInvocationRecord> invocations, String rrscLabel) {
		super(invocations);
		this.backLink = backLink;
		this.rrscLabel = rrscLabel;
		this.calculate = false;
	}

	/**
	 * Whether the action is valid and should be triggered or not.
	 * 
	 * @param calculate
	 *            <ul>
	 *            <li>true - calculation of the RRS is started as background
	 *            process</li>
	 *            <li>false - calculation of the RRS is NOT started at all</li>
	 *            </ul>
	 */
	public RebateRecordAction<ATTACHEDTO> setCalculate(Boolean calculate) {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".setCalculate", calculate));
		this.calculate = Boolean.TRUE.equals(calculate) ? true : false;
		return this;
	}
	
	public ATTACHEDTO back() {
		invocations.add(MethodInvocationRecord.of(getObjectName() + ".back"));
		return backLink;
	}

	@Override
	public String getObjectName() {
		return backLink.getObjectName() + ".RebateRecordAction";
	}

	
	public String getRebateRecordSetCalculationLabel() { return rrscLabel; }
	
	public Boolean getCalculate() {	return calculate; }
	@Override public String toString() {
		return "RebateRecordAction [rrscLabel=" + rrscLabel	+ ", calculate=" + calculate + ", backLink=" + backLink + ", hashCode()=" + hashCode() + "]";
	}

}