package net.pricefx.common.apibuilder.chart;

import net.pricefx.common.api.chart.ChartType;
import net.pricefx.common.api.chart.WaterfallColumnType;
import net.pricefx.common.apibuilder.AbstractBuilder;
import net.pricefx.common.apibuilder.AbstractOptions;
import net.pricefx.common.apibuilder.AbstractSeries;
import net.pricefx.common.apibuilder.IdGenerator;
import net.pricefx.common.apibuilder.Invocations;
import net.pricefx.common.apibuilder.MethodInvocation;
import net.pricefx.common.apibuilder.ObjectIdentity;
import net.pricefx.common.apibuilder.chart.WaterfallChart.Aggregation;

public class WaterfallComparisonChart extends AbstractBuilder {

	public WaterfallComparisonChart(String instanceId, String classId, ObjectIdentity parentChartObjectIdentity, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
		super(instanceId, classId, idGenerator, invocations);
	}

	private WaterfallComparisonOptions options;
	private Dictionary<WaterfallComparisonChart> dictionary;
	
	public WaterfallComparisonSeries addSeries() {
		String id = addInvocation("addSeries");
		return new WaterfallComparisonSeries(this, id, "Series", idGenerator, invocations);
	}
	
	public WaterfallComparisonOptions getOptions() {
		String id = addInvocation("getOptions");
		if (options == null) 
			options = new WaterfallComparisonOptions(this, id, "Options", idGenerator, invocations);
		return options;
	}
	
	public Dictionary<WaterfallComparisonChart> getDictionary() {
		String id = addInvocation("getDictionary");
		if (dictionary == null) 
			dictionary = new Dictionary<>(this, id, "Dictionary");
		return dictionary;
	}
	
	@Override
	public ChartBuilderResult build() {
		addInvocation(METHOD_BUILD);

		ChartBuilderResult r = new ChartBuilderResult();
		r.invocations = invocations.get();
		r.additionalInfo = ChartType.WATERFALLCOMPARISON;
		return r;
	}

	
	public static class WaterfallComparisonOptions extends AbstractOptions<WaterfallComparisonChart,WaterfallComparisonOptions> {

		public WaterfallComparisonOptions(WaterfallComparisonChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}
		
	}
	
	public static class WaterfallComparisonSeries extends AbstractSeries<WaterfallComparisonChart,WaterfallComparisonSeries> {

		public WaterfallComparisonSeries(WaterfallComparisonChart backLink, String instanceId, String classId, IdGenerator idGenerator, Invocations<MethodInvocation> invocations) {
			super(instanceId, classId, backLink, idGenerator, invocations);
		}


		private Aggregation<WaterfallComparisonSeries> aggregation;
		
		public WaterfallComparisonSeries setHideDataLabels(Boolean hideDataLabels) {
			addInvocation("setHideDataLabels",hideDataLabels);
			return this;
		}
		

		public Aggregation<WaterfallComparisonSeries> getAggregation() {
			
			String id = addInvocation("getAggregation");
			if (aggregation == null) 
				aggregation = new Aggregation<WaterfallComparisonSeries>(this, id, "Aggregation", idGenerator, invocations);
			return aggregation;
		}
		
		public WaterfallComparisonSeries addWaterfallColumn(WaterfallColumnType columnType, String source, String label) {
			
			addInvocation("addWaterfallColumn",columnType, source, label);
			return this;
		}
		
		public Formula<WaterfallComparisonSeries> setFormula() {
			
			String id = addInvocation("setFormula");
			return new Formula<WaterfallComparisonSeries>(this, id, "Formula");
		}
		
		public WaterfallComparisonSeries setGeneratedQueryDto(String generatedQueryDto) {
			
			addInvocation("setGeneratedQueryDto",generatedQueryDto);
			return this;
		}
		
	}
	
}