package net.pricefx.workflowengine;

import java.util.Date;
import java.util.EnumSet;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonIgnore;
import net.pricefx.workflowengine.WorkflowStep.WorkflowStepExecutionStatus;


public interface WorkflowStepHistory {

	String HISTORY = "history";
	
	void add(WorkflowStepRecord record);
	Date getLastAccessDate();

	WorkflowStepExecutionStatus getLastStatusForGroup(String groupTypedId);

	WorkflowStepExecutionStatus getLastStatusForUser(String userTypedId);

	public interface WorkflowStepRecord {
		String STATUS = "status";
		String DATE = "date";
		String USERTYPEDID = "userTypedId"; // user that executed the step (with active delegation, this might be different user that one that was selected initially
		String USERLOGINNAME = "userLoginName";
		String DIRECTACTIONEMAIL = "directActionEmail";
		String USERGROUPTYPEDID = "userGroupTypedId";
		String USERGROUPUNIQUENAME = "userGroupUniqueName";
		String SOURCEUSERTYPEDID = "sourceUserTypedId"; // user defined in wf that posses rights to execute
		String SOURCEUSERLOGINNAME = "sourceUserLoginName";
		String WORKFLOWDELEGATIONUNIQUENAME = "workflowDelegationUniqueName";
		String COMMENT = "comment"; // comments can be overwritten, hence storing then also in the history

		WorkflowStepExecutionStatus getStatus();
		Date getDate();
		String getUserTypedId();
		String getUserLoginName();
		String getDirectActionEmail();
		String getUserGroupTypedId();
		String getComment();
		String getSourceUserTypedId();

		@JsonIgnore
		String getRawSourceUserTypedId();

		String getWorkflowDelegationUniqueName();
		String getUserGroupUniqueName();
		String getSourceUserLoginName();

		void setUserLoginName(String loginName);

		void setSourceUserLoginName(String loginName);

		void setUserGroupUniqueName(String uniqueName);
	}

	List<WorkflowStepRecord> getLatestExecutionRecords(WorkflowStepExecutionStatus workflowStepExecutionStatus);

	List<WorkflowStepRecord> getApprovedExecutionRecords();

	List<WorkflowStepRecord> getApprovedOrDeniedOrWithdrawnExecutionRecords();

	WorkflowStepRecord getLastRecordForStatus(EnumSet<WorkflowStepExecutionStatus> of);
	String getLastDirectActionEmailForStatus(EnumSet<WorkflowStepExecutionStatus> of);
}
